from django.db import models
from django.contrib.auth.models import User
from django.db.models import signals
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
import re

# Create your models here.
# 
# Todo:
# - Should channels have their own models? Useful if we intend to user per channel permissions
# - Will need to figure out how to assign events for ops who may not have logins (like freenode staffers)
# - Determine how kickbans will be logged
# - Take a look at django's built-in comment model

class Issue(models.Model):
    create_date = models.DateTimeField(auto_now_add = True,editable = False)
    modify_date = models.DateTimeField(auto_now = True, editable = False)
    closed_date = models.DateTimeField(null = True)
    author = models.ForeignKey(User,null = True)
    comments = generic.GenericRelation('Comment')
    def is_closed(self):
        return self.closed_date != None

class Event(models.Model):
    "Defines the Event base class. Every IRC event class should inherit from this."
    create_date = models.DateTimeField(auto_now_add = False)
    modify_date = models.DateTimeField(auto_now = True) 
    channel = models.CharField(max_length=255)
    log = models.ForeignKey('Log')
    operator = models.ForeignKey('Operator')
    hostmask = models.CharField(max_length=255)
    comments = generic.GenericRelation('Comment') 
    
    def __unicode__(self):
        return self.hostmask
   
    def nick(self):
        return self.hostmask.split('!')[0]
    
    
class Ban(Event):
    mask = models.CharField(max_length=255)
    removal_date = models.DateTimeField(null = True)
    removed_by = models.ForeignKey('Operator', null = True)
    
    
    def is_removed(self):
        return self.removal_date != None

    def __unicode__(self):
        return self.mask

    def ban_pattern(self):
        return re.compile(self.mask.replace('?','.').replace('*','.*'))
   
class Kick(Event):
    reason = models.CharField(max_length=255,null = True) 
    def __unicode__(self):
        return self.hostmask
    
class Mark(Event):
    reason = models.CharField(max_length=255)
    def __unicode__(self):
        return self.reason

class Log(models.Model):
    create_date = models.DateTimeField(auto_now_add = True)
    channel = models.CharField(max_length=255,null = True)
    text = models.TextField()

    def is_private_message(self):
        return self.channel == None

    def __unicode__(self):
        return self.text

class UserProfile(models.Model):
    user = models.ForeignKey(User, unique = True, editable=False)
    hostmask = models.CharField(max_length=255) # Will need to figure out how to lock this down, so that people cannot assign themselves *!*@*
    
    def hostmask_pattern(self):
        return re.compile(self.hostmask.replace('?','.').replace('*','.*'))  
    
    def __unicode__(self):
        return self.user.username
    
class Operator(models.Model):
    user = models.ForeignKey(User, null = True)
    host = models.CharField(max_length=255, null = True)
    def __unicode__(self):
        if self.host:
            return self.host
        else:
            return ''
            
def user_post_save(sender, instance, **kwargs):
    # This automatically adds our UserProfile to newly created or saved Users
    profile, new = UserProfile.objects.get_or_create(user=instance)
models.signals.post_save.connect(user_post_save, sender=User)
    
class Comment(models.Model):
    create_date = models.DateTimeField(null=True)
    modify_date = models.DateTimeField(auto_now = True)
    author = models.ForeignKey('Operator')
    text = models.TextField()
    
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    
    def __unicode__(self):
        return self.text
